@@ -16,15 +16,23 @@
 # under the License.
 from __future__ import annotations
 
-from airflow.plugins_manager import AirflowPlugin
+from airflow.providers.common.compat.sdk import AirflowPlugin
 from airflow.providers.openlineage import conf
-from airflow.providers.openlineage.plugins.listener import get_openlineage_listener
-from airflow.providers.openlineage.plugins.macros import (
-    lineage_job_name,
-    lineage_job_namespace,
-    lineage_parent_id,
-    lineage_run_id,
-)
+
+# Conditional imports - only load expensive dependencies when plugin is enabled
+if not conf.is_disabled():
+    from airflow.providers.common.compat.sdk import HookLineageReader
+    from airflow.providers.openlineage.plugins.listener import get_openlineage_listener
+    from airflow.providers.openlineage.plugins.macros import (
+        lineage_job_name,
+        lineage_job_namespace,
+        lineage_parent_id,
+        lineage_root_job_name,
+        lineage_root_job_namespace,
+        lineage_root_parent_id,
+        lineage_root_run_id,
+        lineage_run_id,
+    )
 
 
 class OpenLineageProviderPlugin(AirflowPlugin):
@@ -37,10 +45,17 @@ class OpenLineageProviderPlugin(AirflowPlugin):
 
     name = "OpenLineageProviderPlugin"
     if not conf.is_disabled():
-        macros = [lineage_job_namespace, lineage_job_name, lineage_run_id, lineage_parent_id]
+        macros = [
+            lineage_job_namespace,
+            lineage_job_name,
+            lineage_run_id,
+            lineage_parent_id,
+            lineage_root_run_id,
+            lineage_root_job_name,
+            lineage_root_job_namespace,
+            lineage_root_parent_id,
+        ]
         listeners = [get_openlineage_listener()]
-        from airflow.lineage.hook import HookLineageReader
-
         hook_lineage_readers = [HookLineageReader]
     else:
         macros = []
